#pragma once
#ifndef _BOUNDSCHECK_H
#define _BOUNDSCHECK_H

/******************************************************
 * BOUNDS CHECKING MACROS                             *
 *                                                    *
 * Checks value to be between min and max             *
 * return is value if check is successful, and err    *
 * if check fails                                     *
 *                                                    *
 * The checks are only made if the symbol:            *
 * BOUNDS_CHECK_ALL                                   *
 * has been defined, else the macro evaluates to      *
 * value without any additional overhead              *
 ******************************************************/


#ifdef BOUNDS_CHECK_ALL
#define BOUNDSCHECK(value,min,max,err) ( (value >= min) && (value <= max)) ? value : err
#else
#define BOUNDSCHECK(value,min,max,err) value
#endif

#endif /* _BOUNDSCHECK_H */
