#ifndef DAQRAWTRIGGERMASKV0_H
#define DAQRAWTRIGGERMASKV0_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/BitFields.h"
#include "DAQDataFormats/TriggerDefines.h"
#include "DAQDataFormats/Macros.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawTriggerMask
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE triggermask
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

  namespace  VERSION_NAMESPACE {
        namespace CURRENT_FIELDS_SUB_NAMESPACE {

            static const uint32_t TRIGGERMASK_SIZE = 4;

            enum TriggerMaskMASKS{
                MASK_MonteCarloFlag   = 0x80000000,
                MASK_TrigMod          = 0x3F000000,
                MASK_TrigMask_Low     = 0x00FFFFFF,
                MASK_TrigMask_High    = 0xFFFFFFFF,
                MASK_TrigMask_High2   = 0xFFFFFFFF,
                MASK_Prescale         = 0x0000FFFF,
                MASK_Version          = 0xFFFF0000
            };
  
            enum TriggerMaskWORDS{
                WORD_MonteCarloFlag   = 0,
                WORD_TrigMod          = 0,
                WORD_TrigMask_Low     = 0,
                WORD_TrigMask_High    = 1,
                WORD_TrigMask_High2   = 2,
                WORD_Prescale         = 3,
                WORD_Version          = 3
            };

            DEFINE_SHIFT_FROM_MASK(MonteCarloFlag);
            DEFINE_SHIFT_FROM_MASK(TrigMod);
            DEFINE_SHIFT_FROM_MASK(TrigMask_Low);
            DEFINE_SHIFT_FROM_MASK(TrigMask_High);
            DEFINE_SHIFT_FROM_MASK(TrigMask_High2);
            DEFINE_SHIFT_FROM_MASK(Prescale);
            DEFINE_SHIFT_FROM_MASK(Version);
        }// end of namespace triggermask
  

  //! Class to hold the data corresponding to the trigger mask
  /*!
    The RawTriggerMaskV0 contains the information on the type of trigger
    that is being issued.

    The class supports general initialization with an internal or external
    data buffer as well as copyin style initialization for use with
    STL containers.
  */
  class RawTriggerMask: public PREVIOUS_VERSION_CLASS_NAME {
    
        public:
      
            RawTriggerMask (const version_t); ///< The default constructor
            RawTriggerMask (const RawTriggerMask&);
            ~RawTriggerMask(); ///< Destructor

            void     init      ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print     (std::ostream& os=std::cout) const;
            uint32_t sizeofdata() const { return CURRENT_FIELDS_SUB_NAMESPACE::TRIGGERMASK_SIZE;}
            /// Nanoslice of version 0 is a fixed size format
            using    RawDAQData::readData;
            void*    readData(const void* pointer){return RawDAQData::readDataGeneral(pointer);};

            IMPLEMENT_FIRST_GET_AND_SET(Version       , uint32_t);
            IMPLEMENT_FIRST_GET_AND_SET(MonteCarloFlag, uint32_t);
            IMPLEMENT_FIRST_GET_AND_SET(TrigMod       , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TrigMask_Low  , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TrigMask_High , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TrigMask_High2, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(Prescale      , uint32_t)

	    INIT_FIRST(getTriggerMask, const, uint32_t, )
            INIT_FIRST(setTriggerMask,      , bool    , const uint32_t)

  };
  
  
}} // END NAMESPACE daqdataformats
#endif /* DAQRAWTRIGGERMASKV0_H */
