#ifndef _DBUTILS_DAQCONFIG_DDTCONNECTCONFIG_H
#define _DBUTILS_DAQCONFIG_DDTCONNECTCONFIG_H

#include <DatabaseUtils/SimpleParameterSet.h>
#include <DatabaseUtils/DAQConfig/ConfigDataTree.h>
#include <DatabaseUtils/DAQConfig/GlobalNameSpec.h>
#include <DatabaseUtils/DAQConfig/SubsystemNameSpec.h>
#include <boost/shared_ptr.hpp>
#include <iostream>
#include <map>
#include <string>
#include <vector>

namespace dbutils {
namespace daqconfig {

/**
 * Worker class to handle fetching of DDT Connection
 * configuration data.
 *
 * @author Kurt Biery
 * @version $Revision: 1.1 $ $Date: 2013/12/06 22:56:35 $
 */

class DDTConnectConfig {

 public:

    // ********************
    // *** Constructors ***
    // ********************

    DDTConnectConfig(const std::vector<int64_t>& subsystemConfigIdList);

    DDTConnectConfig(const SubsystemNameSpec& nameSpec);

    DDTConnectConfig(const GlobalNameSpec& nameSpec);

    DDTConnectConfig(const int64_t& globalConfigId);

    virtual ~DDTConnectConfig() {};

    // *****************
    // *** Accessors ***
    // *****************

    virtual bool hasValidData() const {return _hasValidData;}
    int64_t getPrimarySubsystemConfigId() const {return _primarySubsystemConfigId;}
  
 protected:

    bool _fetchAllData(const std::vector<int64_t>& subsystemConfigIdList);

    bool _hasValidData;
    int64_t _primarySubsystemConfigId;

    boost::shared_ptr<ConfigDataTree> _ddtConfigDataTree;
};

};  // end of namespace daqconfig
};  // end of namespace dbutil

#endif
