#ifndef _DBUTILS_RUNHISTORY_H
#define _DBUTILS_RUNHISTORY_H

#include <string>
#include <vector>
#include <map>
#include <ctime>
#include <stdint.h>  /* uint64_t */

#include <NovaDatabase/Util.h>
#include <NovaDatabase/Table.h>
#include <DatabaseUtils/DAQConfig/PixelEnableMasks.h>
#include <DatabaseUtils/DAQConfig/PixelThresholds.h>

namespace db = nova::database;
namespace daqcfg = dbutils::daqconfig;

namespace dbutils {

/**
 * Utilities for accessing DB via kerberos
 *
 * @author Jonathan Paley
 * @version $Revision: 1.9 $ $Date: 2013/02/11 15:36:10 $
 */

  class RunHistory {
    
  public:

    class FEB {
    public:
      FEB() {};
      ~FEB() {};

      std::string hwid;
      std::string dcmid;
      int loc;
      int nActivePix;
      int pixelMask;
      int pixelThresh[32];
      int apdV;
      bool isEnabled;
      bool isCooled;
      bool tempEnabled;
      float temperature;
    };

    class DCM {
    public:
      DCM() {};
      ~DCM() {};

      std::string hwid;
      std::string appname;
      int64_t febMask;
      std::map<int,FEB> feb;
    };

    class BNEVB {
    public:
      BNEVB() {};
      ~BNEVB() {};

      std::string hwid;
      std::string appname;
    };

    RunHistory(int run, std::string det);
    RunHistory(int run, int detid);
    ~RunHistory();

    void Load();
    void SetDBInfo(std::string dbname="", std::string dbhost="", 
		   std::string user="",   std::string port="");
    void SetRunNumber(int run) {_runNumber = run; }
    void SetDetector(std::string det);

    void Dump();

    int  RunNumber() const { return _runNumber; }
    int  NSubruns()  const { return _nsubrun; }
    std::string Detector() const { return _detector; }

    time_t TStart() const;
    time_t TStop() const;
    std::string TStartAsString() const { return _tStart;}
    std::string TStopAsString() const { return _tEnd;}
      
    int Duration() const { return TStop()-TStart(); }  ///< in units of seconds
    int NTriggers() const { return _nTotTrig; }

    int NDCMs() const { return _dcmList.size(); }
    int NBNEVBs() const { return _bnevbList.size(); }
    int NActiveChannels() const { return _nActiveChannels; }
    int NTotalChannels() const { return _nTotalChannels; }
    int NTotalFEBs() const { return _nTotalFEBs; }
    int NMaskedOffFEBs() const { return _nMaskedOffFEBs; }
    const DCM GetDCM(int idcm) { return _dcmList[idcm]; }
    
    int NDCMInstalled() const { return _nInstalledDCMs; }
    int NFEBInstalled() const { return _nInstalledFEBs; }
    int NAPDInstalled() const { return _nInstalledAPDs; }

    bool FEBEnableMask(const std::string& dcmName, int64_t& febMask);
    bool PixelEnableMask(const std::string& dcmName,
                         const int& febId, int32_t& pixelMask);
    bool PixelThresholdsForFEB(const std::string& dcmName,
                               const int& febId,
                               std::vector<int16_t>& thresholdList);

  private:

    int _runNumber;
    int _nsubrun;
    int _nTotTrig;
    int _nActiveChannels;
    int _nTotalChannels;
    int _nTotalFEBs;
    int _nMaskedOffFEBs;
    int _nInstalledDCMs;
    int _nInstalledFEBs;
    int _nInstalledAPDs;
    
    std::string _detector;
    std::string _tStart;
    std::string _tEnd;
    std::vector<int> _nEvtTrig;
    std::vector<DCM> _dcmList;
    std::vector<BNEVB> _bnevbList;

    int64_t _gCfgId;

    std::auto_ptr<db::Table> runsTable;
    std::auto_ptr<db::Table> resTable;
    std::auto_ptr<db::Table> hwposTable;
    std::auto_ptr<daqcfg::PixelEnableMasks> _pixMaskPtr;
    std::auto_ptr<daqcfg::PixelThresholds>  _pixThreshPtr;

  };  // end of class Krb5
};  // end of namespace dbutils

#endif
