#include "DAQDataFormats/RawRunHeader.h"
#include "DAQDataFormats/DataFormatException.h"

// It is requred to
//   1. include DAQDataFormats/DataFormatFirst.inc
//   2. define CURRENT_CLASS
//   3.    define CURRENT_CLASS_VERSION for version of the class
//      OR define LAST_CLASS_VERSION    for the main structure of the class
//   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
//   5. include DAQDataFormats/DataFormatLast.inc
// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawRunHeader
#define  LAST_CLASS_VERSION           0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawrunhdr
#include "DAQDataFormats/DataFormatLast.inc"


namespace daqdataformats{

//------------------------Constructors------------------------------------------
/// After this constructor the buffer pointer is set to the internal source
RawRunHeader::RawRunHeader(const version_t version):
     PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    /// Apply the default constructor
    defaultConstructorFixedSize();
}

//------------------------------------------------------------------------------
RawRunHeader::RawRunHeader(const RawRunHeader& copy_in):
    PREVIOUS_VERSION_CLASS_NAME(copy_in)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    lastOperationsInDeepCopy(copy_in);
}

//------------------------------------------------------------------------------
RawRunHeader::RawRunHeader(const void* buffer, const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    /// Use default Shallow copy constructor
    copy(buffer);
}

//------------------------------------------------------------------------------
RawRunHeader::~RawRunHeader(){ }


//------------------------------------------------------------------------------
/// All binding routines
#define STANDARD_ROUTINES                                                 \
            BIND_GET_AND_SET(MarkerHi              , version0, uint32_t);\
            BIND_GET_AND_SET(MarkerLo              , version0, uint32_t);\
            BIND_GET_AND_SET(Reserved1             , version0, uint32_t);\
            BIND_GET_AND_SET(Reserved2             , version0, uint32_t);\
            BIND_GET_AND_SET(Version               , version0, uint32_t);\
            BIND_GET_AND_SET(RunNumber             , version0, uint32_t);\
            BIND_GET_AND_SET(RunType               , version0, uint32_t);\
            BIND_GET_AND_SET(ConfigIDX             , version0, uint32_t);\
            BIND_GET_AND_SET(Partition             , version0, uint32_t);\
            BIND_GET_AND_SET(Subrun                , version0, uint32_t);\
            BIND_GET_AND_SET(ConfigurationBlockSet , version0, uint32_t);\
            BIND_GET_AND_SET(RunControlVersion     , version0, uint32_t);\
            BIND_GET_AND_SET(RunControlID          , version0, uint32_t);\
            BIND_GET_AND_SET(DataLoggerVersion     , version0, uint32_t);\
            BIND_GET_AND_SET(DataLoggerID          , version0, uint32_t);\
            BIND_GET_AND_SET(RunStartTime          , version0, uint32_t);\
            BIND_GET_AND_SET(SubRunStartTime       , version0, uint32_t);\
            BIND_GET_AND_SET(SubRunEndTime         , version0, uint32_t);\
            BIND_GET_AND_SET(RunEndTime            , version0, uint32_t);\
            BIND_GET_AND_SET(TotalEvents           , version0, uint32_t);\
            BIND_GET_AND_SET(UniversalEventStartLo , version0, uint32_t);\
            BIND_GET_AND_SET(UniversalEventStartHi , version0, uint32_t);\
            BIND_GET_AND_SET(UniversalEventEndLo   , version0, uint32_t);\
            BIND_GET_AND_SET(UniversalEventEndHi   , version0, uint32_t);\
            BIND_GET_AND_SET(RunSize               , version0, uint32_t);\
            BIND_GET_AND_SET(TriggerVersion        , version0, uint32_t);\
            BIND_GET_AND_SET(TriggerCtrlID         , version0, uint32_t);\
            BIND_GET_AND_SET(TriggerMC             , version0, uint32_t);\
            BIND_GET_AND_SET(VTMod                 , version0, uint32_t);\
            BIND_GET_AND_SET(ValidTriggerTypesLow  , version0, uint32_t);\
            BIND_GET_AND_SET(ValidTriggerTypesHigh , version0, uint32_t);\
            BIND_GET_AND_SET(ValidTriggerTypesHigh2, version0, uint32_t);\
            BIND_GET_AND_SET(TriggerListIDX        , version0, uint32_t);\
            BIND_GET_AND_SET(TriggerPrescaleListIDX, version0, uint32_t);\
            BIND_GET_AND_SET(CRC                   , version0, uint32_t);\
            BIND_CUSTOM_FUNCTION(getUniversalEventStart, version0, const ,uint64_t, ); \
            BIND_CUSTOM_FUNCTION(setUniversalEventStart, version0,       ,bool    , const uint64_t);\
            BIND_CUSTOM_FUNCTION(getUniversalEventEnd  , version0, const,uint64_t, ); \
            BIND_CUSTOM_FUNCTION(setUniversalEventEnd  , version0,      ,bool    , const uint64_t);\
            BIND_CUSTOM_FUNCTION(setTail               , version0,      ,bool, ); \
            BIND_CUSTOM_FUNCTION(close                 , version0,      ,bool, ); \
            BIND_CUSTOM_FUNCTION(advanceEventNumber    , version0,      ,bool, ); \
            BIND_CUSTOM_FUNCTION(addValidTriggerTypesLow  , version0,   ,bool    , const uint32_t);\
            BIND_CUSTOM_FUNCTION(addValidTriggerTypesHigh , version0,   ,bool    , const uint32_t);\
            BIND_CUSTOM_FUNCTION(addValidTriggerTypesHigh2, version0,   ,bool    , const uint32_t);\
            BIND_CUSTOM_FUNCTION(addRunSize               , version0,   ,bool    , const uint32_t);\
            BIND_STADARD_FUNCTION(sizeofdata);\
            BIND_STADARD_FUNCTIONS;

#include "DAQDataFormats/setFunctionPointers_template.inc"

}// end of namespace
