#include "DAQDataFormats/RawSummaryEventHeader.h"
#include "DAQDataFormats/DataFormatException.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawSummaryEventHeader
#define  LAST_CLASS_VERSION           0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawsummaryeventhdr
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{

//------------------------Constructors------------------------------------------
/// After this constructor the buffer pointer is set to the internal source
RawSummaryEventHeader::RawSummaryEventHeader(const version_t version):
     PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    /// Apply the default constructor
    defaultConstructorFixedSize();
}

//------------------------------------------------------------------------------
RawSummaryEventHeader::RawSummaryEventHeader(const RawSummaryEventHeader& copy_in):
    PREVIOUS_VERSION_CLASS_NAME(copy_in)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    lastOperationsInDeepCopy(copy_in);
}

//------------------------------------------------------------------------------
RawSummaryEventHeader::RawSummaryEventHeader(const void* buffer, const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    /// Use default Shallow copy constructor
    copy(buffer);
}

//------------------------------------------------------------------------------
RawSummaryEventHeader::~RawSummaryEventHeader(){ }


//------------------------------------------------------------------------------
/// All binding routines
#define STANDARD_ROUTINES                                          \
            BIND_GET_AND_SET(MarkerLeft      , version0, uint32_t);\
            BIND_GET_AND_SET(MarkerRight     , version0, uint32_t);\
            BIND_GET_AND_SET(Version         , version0, uint32_t);\
            BIND_GET_AND_SET(DataLoggerID    , version0, uint32_t);\
            BIND_GET_AND_SET(RunNumber       , version0, uint32_t);\
            BIND_GET_AND_SET(RunType         , version0, uint32_t);\
            BIND_GET_AND_SET(ConfigIDX       , version0, uint32_t);\
            BIND_GET_AND_SET(Partition       , version0, uint32_t);\
            BIND_GET_AND_SET(Subrun          , version0, uint32_t);\
            BIND_GET_AND_SET(EventNumberLo   , version0, uint32_t);\
            BIND_GET_AND_SET(EventNumberHi   , version0, uint32_t);\
            BIND_GET_AND_SET(EventType       , version0, uint32_t);\
            BIND_GET_AND_SET(EventSize       , version0, uint32_t);\
            BIND_GET_AND_SET(DataBlockNumber , version0, uint32_t);\
            BIND_GET_AND_SET(MonteCarloFlag  , version0, bool);\
            BIND_GET_AND_SET(TriggerFilled   , version0, uint32_t);\
            BIND_GET_AND_SET(TailFilled      , version0, uint32_t);\
            BIND_IS_AND_SET      (CRCCalculationUsed, version0);\
            BIND_IS_AND_SET      (EventIncomplete, version0);\
            BIND_CUSTOM_FUNCTION(addEventSize          , version0, ,bool       , const uint32_t);\
            BIND_CUSTOM_FUNCTION(advanceEventNumber    , version0, ,bool, ); \
            BIND_CUSTOM_FUNCTION(advanceDataBlockNumber, version0, ,bool, ); \
            BIND_GET_AND_SET(EventNumber, version0, uint64_t);\
            BIND_GET(TailVersion, version0, version_t);\
            BIND_STADARD_FUNCTIONS \
            BIND_STADARD_FUNCTION(sizeofdata);

#include "DAQDataFormats/setFunctionPointers_template.inc"


}// end of namespaces
